const width = 800;
const height = 500;

const mapWidth = width * 0.65;
const plotWidth = width - mapWidth + 100;

const margin = 60;

const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', mapWidth)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', plotWidth)
  .attr('height', height);

const tooltip = d3
  .select('#my_dataviz')
  .append('div')
  .attr('id', 'tooltip');

const mapColorAttr = 'Quantity';

const plotColorAttr = 'Continent';
const xAttr = 'Year';
const yAttr = 'Quantity';

var xExtent = undefined;
var yExtent = undefined;
var xScale = d3.scaleLinear();
var yScale = d3.scaleLinear();
var xScaleType = 'linear';
var yScaleType = 'linear';

function getDataExtent(data, attr) {
  return d3.extent(
    d3.map(data, (r) => +r[attr]).filter((r) => r > 0),
  );
}

function render(mapData, countryData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([mapWidth, height], mapData);
  var path = d3.geoPath(projection);
  console.log(countryData.values());
  var mapColorScale = d3
    .scaleThreshold()
    .domain([1000, 5000, 10000, 20000, 30000, 40000, 50000])
    .range([
      '#e7e1ef',
      '#d4b9da',
      '#c994c7',
      '#df65b0',
      '#e7298a',
      '#ce1256',
      '#91003f',
    ]);

  xExtent = getDataExtent(countryData.values(), xAttr);
  xScale = d3
    .scaleLinear()
    .domain([1995, 2023])
    .range([margin, plotWidth - margin]);

  yExtent = getDataExtent(countryData.values(), yAttr);
  yScale = d3
    .scaleLinear()
    .domain([1000, 60000])
    .range([height - margin, margin]);

  const plotColorScale = d3.scaleOrdinal(
    d3.schemeCategory10,
  );

  function getCountryColor(d) {
    let entry = countryData.get(d.properties.name);
    if (entry == undefined) {
      return 'grey';
    }
    let lastProd =
      entry.Production[entry.Production.length - 1];
    return entry == undefined
      ? 'grey'
      : mapColorScale(lastProd.Quantity);
  }

  function getPointColor(d) {
    return plotColorScale(d[plotColorAttr]);
  }

  const countryMarks = svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    .attr('fill', getCountryColor)
    .attr('id', (d) => d.properties.name)
    .attr('class', 'country')
    .classed('poly', true)
    .attr('d', path);

  svg2
    .append('g')
    .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
    .call(
      d3
        .axisBottom(xScale)
        .tickFormat(d3.format('d'))
        .ticks(7),
    );

  svg2
    .append('text')
    .text(xAttr)
    .attr('x', plotWidth / 2)
    .attr('y', 475)
    .style('text-anchor', 'middle');

  svg2
    .append('g')
    .attr('id', 'yAxisG')
    .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
    .call(d3.axisLeft(yScale));

  svg2
    .append('text')
    .text(yAttr)
    .attr('transform', 'rotate(-90)')
    .attr('x', -height / 2)
    .attr('y', 12);

  xScale.domain([1995, 2023]);
  yScale.domain([1000, 60000]);

  const lineGenerator = d3
    .line()
    .x((row) => xScale(row.year))
    .y((row) => yScale(row.Quantity));

  var res = countryData.keys();
  console.log(res);
  var color = d3
    .scaleOrdinal()
    .domain(res)
    .range([
      '#cb181d',
      '#41ab5d',
      '#fe9929',
      '#4292c6',
      '#08519c',
      '#6a51a3',
      '#dd3497',
      '#fc9272',
      '#006d2c',
      '#810f7c',
      '#99d8c9',
      '#f1b6da',
      '#ffeda0',
    ]);

  let lineData = new Map();
  countryData.forEach((data, country) => {
    let include = false;
    let prod = data.Production;
    prod.forEach((production) => {
      let quantity = production.Quantity;
      if (quantity > 5000) {
        include = true;
      }
    });
    if (include && !lineData.has(data.Country)) {
      lineData.set(data.Country, {
        Country: data.Country,
        Production: data.Production,
        Continent: data.Continent,
      });
    }
  });

  console.log(lineData);

  const scatterMarks = svg2
    .selectAll('.line')
    .data(lineData.values())
    .enter()
    .append('path')
    .attr('id', (d) => d.Country)
    .attr('class', 'line')
    .attr('fill', 'none')
    .attr('stroke-width', 1.5)
    .attr('stroke', function (d) {
      return color(d.Country);
    })
    .attr('d', function (d) {
      return lineGenerator(d.Production);
    });

  function popUpTooltip(event, data) {
    if (
      countryData.get(data.properties.name) == undefined
    ) {
      tooltip.style('opacity', 1);
      tooltip.html(`no data`);
    } else {
      let country = countryData.get(data.properties.name);
      tooltip.style('opacity', 1);
      tooltip.html(
        `${data.properties.name.toUpperCase()}<br/>Production: ${country.Production[country.Production.length - 1].Quantity}`,
      );
    }
  }

  function linePopUpTooltip(event, data) {
    let country = countryData.get(data.Country);
    tooltip.style('opacity', 1);
    tooltip.html(
      `${data.Country.toUpperCase()}<br/>Production: ${country.Production[country.Production.length - 1].Quantity}`,
    );
  }

  function adjustTooltip(event) {
    tooltip
      .style('left', event.x - 80 + 'px')
      .style('top', event.y - 80 + 'px');
  }

  countryMarks
    .on('mouseover', function (event, data) {
      console.log(countryData.get(data.properties.name));
      popUpTooltip(event, data);
      console.log(this);
      d3.selectAll(`#${this.id}.line`)
        .attr('stroke-width', 3)
        .attr('stroke', 'black');
      d3.selectAll(`#${this.id}.country`)
        .attr('stroke-width', 1)
        .attr('stroke', 'black');
    })
    .on('mousemove', function (event, data) {
      adjustTooltip(event);
    })
    .on('mouseleave', function (event, data) {
      tooltip.style('opacity', 0);
      d3.selectAll(`#${this.id}.line`)
        .attr('stroke-width', 1.5)
        .attr('stroke', function (d) {
          return color(d.Country);
        });
      d3.selectAll(`#${this.id}.country`).attr(
        'stroke-width',
        0,
      );
    });

  scatterMarks
    .on('mouseover', function (event, data) {
      console.log('hello');
      linePopUpTooltip(event, data);
      d3.selectAll(`#${this.id}.line`)
        .attr('stroke-width', 3)
        .attr('stroke', 'black');
      d3.selectAll(`#${this.id}.country`)
        .attr('stroke-width', 1)
        .attr('stroke', 'black');
    })
    .on('mouseleave', function (event, data) {
      tooltip.style('opacity', 0);
      d3.selectAll(`#${this.id}.line`)
        .attr('stroke-width', 1.5)
        .attr('stroke', function (d) {
          return color(d.Country);
        });
      d3.selectAll(`#${this.id}.country`).attr(
        'stroke-width',
        0,
      );
    });

  d3.select('#yAxisG').on('click', (event, data) =>
    toggleAxisScale(),
  );
}

function toggleAxisScale() {
  if (yScaleType == 'linear') {
    yScale = d3
      .scaleLog()
      .domain(yExtent)
      .range([height - margin, margin]);
    yScaleType = 'log';
  } else if (yScaleType == 'log') {
    yScale = d3
      .scaleLinear()
      .domain(yExtent)
      .range([height - margin, margin]);
    yScaleType = 'linear';
  }

  svg2.select('#yAxisG').remove();

  svg2
    .append('g')
    .attr('id', 'yAxisG')
    .on('click', () => toggleAxisScale())
    .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
    .transition()
    .duration(2000)
    .call(d3.axisRight(yScale));
}

const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const countryStatsUrl =
  'https://raw.githubusercontent.com/lpenrice/CP341/main/all_production.csv';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

function parseNumStr(s) {
  return +s.replaceAll(',', '').replaceAll('$$', '');
}

Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
]).then(([mapData, csvData]) => {
  var countryData = new Map();
  var countryToMapNames = new Map([
    ['United States of America', 'USA'],
    [
      'Tanzania, The United Republic of',
      'United Republic of Tanzania',
    ],
    ['United Kingdom', 'England'],
    ['United Kingdom', 'Wales'],
    ['China, mainland', 'China'],
    ['China, Tawiwan Province of', 'Taiwan'],
    ['Guinea-Bissau', 'Guinea Bissau'],
    ['Timor-Leste', 'East Timor'],
    ['Bahamas', 'The Bahamas'],
    ['United Kingdom', 'England'],
    ['Belgium and Luxemburg', 'Luxemburg'],
    ['Republic of North Macedonia', 'Macedonia'],
    ['Serbia', 'Republic of Serbia'],
    ['Republic of Türkiye', 'Turkey'],
    ['Syrian Arab Republic', 'Syria'],
    [
      'Congo (the Democratic Republic of the)',
      'Democratic Republic of the Congo',
    ],
    ["Côte d 'Ivoire", 'Ivory Coast'],
    ['Congo', 'Republic of the Congo'],
    [
      'Tanzania, the United Republic of',
      'United Republic of Tanzania',
    ],
    ['Korea (the Republic of)', 'South Korea'],
    [
      "Korea (the Democratic People's Republic of)",
      'North Korea',
    ],
    ['Brunei Darussalam', 'Brunei'],
    ['French Guiana', 'Suriname'],
  ]);
  csvData = csvData.filter((d) => d.Country != 'Global');
  console.log(csvData);
  csvData.forEach((row) => {
    if (countryToMapNames.has(row.Country)) {
      row.Country = countryToMapNames.get(row.Country);
    }
    row.Country = normalizeCountryName(row.Country);
    row.Quantity = parseInt(
      row.Quantity.replace(/\s/g, ''),
      10,
    );
    row.Continent = row.Continent;

    if (!countryData.has(row.Country)) {
      countryData.set(row.Country, {
        Country: row.Country,
        Production: [],
        Continent: row.Continent,
      });
    }
    countryData.get(row.Country).Production.push({
      year: row.Year,
      Quantity: row.Quantity,
    });
  });
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });
  console.log(countryData);
  render(mapData, countryData);
})